% Matlab code example for Algorithm 4.3 presented in
% "A deterministic sparse FFT algorithm for vectors with small support"
% by Gerlind Plonka and Katrin Wannenwetsch

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RANDOM SAMPLING VECTOR
% (uncomment this code for random construction of a sampling vector)

% set parameters
N = 2^20;         % vector length (must be power of 2)
m = 16;            % support length (should be at most N/4, otherwise use ifft)

% calculate random sampling vector
x=zeros(N,1);                       % allocate space
L=ceil(log2(m));                    % calculate L
firstsuppind=randi(N);              % random first support index
                                    % calculate support
supp=[firstsuppind:min(firstsuppind+m-1,N),1:firstsuppind+m-1-N];
suppzeros=randi(m-2);               % random number of zeros inside support
supp(randperm(m-2,suppzeros)+1)=[]; % delete suppzeros number of support indizes (not first or last)
                                    % set random complex values        
x(supp)=20*(rand(m-suppzeros,1)-0.5+1i*(rand(m-suppzeros,1)-0.5));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % DETERMINISTIC SAMPLING VECTOR
% % (uncomment this code for deterministic construction of a sampling vector)
% 
% % set parameters
% N = 2^10;         % vector length (must be power of 2)
% m = 6;            % support length (should be at most N/4, otherwise use ifft)
% 
% % set vector entries
% x(1)=1;
% x(2)=2;
% x(1021)=3;
% x(1023)=4;
% x(1024)=5;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% create noised Fourier data

SNR = 20;           % set desired SNR

xhat=fft(x);        % Fourier transform

noise=rand(N,1)-0.5+1i*(rand(N,1)-0.5);          % create random noise
noise=norm(xhat)*10^(-SNR/20)/norm(noise)*noise; % scale according to SNR
xhatnoise=xhat+noise;                            % noisy input data

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% apply algorithm to noisy data
xrec=algorithm(xhatnoise,m);



